//Chris Hunter
/******************UNIMAC*****************
This exercise begins to introduce some
control over the medium. The server is
effectively the transmit 1/2 of a full
carrier-sensing MAC. It carrier-senses
and then transmits to each user,
retransmitting when a user fails to ACK
the original transmission. So, the users
are responsible for the receive half of the
full MAC layer. They must filter the
destination addresses of received packets
and ACK received DATA packets to prevent
retransmissions.

Note: The "uni" in "uniMac" only means that
you are controlling the medium in only one
direction. User code is still responsible
for transmitting received Ethernet traffic
to the server. This transmission, however,
will not have the benefit of carrier sensing
and retransmissions.
*****************************************/

#include "warpmac.h"
#include "warpphy.h"
#include "xparameters.h"
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include "xutil.h"
#include "xcache_l.h"
#include "xexception_l.h"
#include "xcache_l.h"
#include "xutil.h"
#include "xexception_l.h"

//Routing table with agreed upon mapping between dipswitches and physical addresses
typedef struct {
	unsigned char addr[6];
} route;

//Byte array containing physical MAC address of this node
unsigned char myAddr[6];
//Instance of the routing table
route routeTable[16];
//Index to the routing table that identifies this node
unsigned char myID;

//Data packet with payload meant for Ethernet transmission
#define DATAPACKET 0
//Acknowedgement packet meant for halting retransmissions
#define ACKPACKET 1


//Buffer for holding a packet-to-xmit across multiple retransmissions
Macframe txBuffer;
//Buffer to hold received packet
Macframe rxBuffer;


void buttonHandler(){
	/*This function is called by the interrupt handler
	upon the depression of a push button. This function
	can be used to print messages over the serial port.*/

	unsigned int test = 15;
	xil_printf("Hello World, the value of test is, %d\r\n",test);

	/////User Code Here/////

	//Print statistics like the total number of received packets, or the total number of bad packets

	///////////////////////

}

int ethernet_callback(Xuint32 length, char* payload){
	/*This function is attached to the underlying framework in the main
	 function of this file. It is executed upon the reception of an Ethernet
	 frame. By the time this function is called, the payload of the Ethernet
	 frame is already being copied to the OFDM PHY transmiter's packet buffer
	 via the internal DMA engine in the Ethernet MAC. All the user is responsible
	 for is creating a header for the over-the-air packet. The length of the received
	 Ethernet frame is passed into this function as an argument*/

	//PSEUDOCODE:
	//1) Fill in the length of the Ethernet frame into the length field of the header of the global transmit buffer.
	//2) Identify the packet type as a DATAPACKET
	//3) Set the modulation order of the packet by filling in the appropriate field in the header. Use QPSK.
	//4) The server is entry 0 of the routing table. Copy this address into the header's destination address.
		//Look at CSMAMAC for an example of how this is done.
	//5) Copy your address into the source address field. Your address is available in the global myAddr array.
	//6) Send the transmit buffer over OFDM

	return 0;
}


int receiveBadPacket(Macframe* packet){
	/*This function is attached to the underlying framework in the main
	 function of this file. It is executed when a packet is received from the
	 OFDM PHY receiver that fails checksum. Since at least one bit was in error,
	 the packet is, in general, no good to us. We use this function to blink
	 LEDs to help us visualize subjectively the good/bad ratio of received
	 packets. Additionally, this function could be used to keep track of the
	 number of failed packets more objectively for rate adaptation schemes.*/

	warpmac_incrementLEDLow();
}

int receiveGoodPacket(Macframe* packet) {
	/*This function is attached to the underlying framework in the main
	 function of this file. It is executed when a packet is received from the
	 OFDM PHY receiver that passes checksum. This function is analogous to the
	 ethernet_callback function, but in the other direction. It pushes the
	 received packet over-the-wire to its sink. Additionally, it transmits an ACK
	 so that the server stops retransmitting. */

	warpmac_incrementLEDHigh();

	//PSEUDOCODE:
	//1) Check if the destination was addressed to you
	//If yes,
	//	1.2) Instantiate a new Macframe for the ackPacket
	//	1.3) Set the length to 0 (no payload)
	//	1.4) Set the type to ACKPACKET
	//	1.5) Set the rate to QPSK
	//	1.6) Copy your address into the source address of the ACK
	//  1.7) Copy the sender's adress into the destination address of the ACK
	//	1.8) Send the ACK over OFDM
	//	1.9) Send the received packet over Ethernet
	//If no,
	//	2) Break from function (i.e. return 0;)

return 0;
}



int main(){

	//Read Dip Switch value from FPGA board.
	//This value will be used as an index into the routing table for other nodes
	myID = warpmac_getMyId();

	//Create an arbitrary address for this node
	unsigned char tmpAddr[6] = {0x16,0x24,0x63,0x53,0xe2,0xc2+myID};

	//Copy this address into a global array for the rest of the code to access
	memcpy(myAddr,tmpAddr,6);

	//Fill an arbitrary routing table so that nodes know each others' addresses
	unsigned char i;
	for(i=0;i<16;i++){
		routeTable[i].addr[0] = myAddr[0];
		routeTable[i].addr[1] = myAddr[1];
		routeTable[i].addr[2] = myAddr[2];
		routeTable[i].addr[3] = myAddr[3];
		routeTable[i].addr[4] = myAddr[4];
		routeTable[i].addr[5] = myAddr[5]+i-myID;
	}

	//Initialize the framework
	warpmac_init();
	//Tell the framework the MAC address of this node
	warpmac_setMacAddr(&myAddr);
	//Tell the framework where to place a received packet
	warpmac_setRxBuffer(&rxBuffer,0);
	//Tell the framework where to send a received packet from
	warpmac_setTxBuffer(1);
 	//Copy MAC address into source field
	memcpy(txBuffer.header.srcAddr,myAddr,6);
	//Attach the framework to the good packet handler
	warpmac_setGoodPacketHandler(receiveGoodPacket);
	//Attach the framework to the bad packet handler
	warpmac_setBadPacketHandler(receiveBadPacket);
	//Attach the framework to the ethernet handler
	warpmac_setEmacHandler(ethernet_callback);
	//Attach the framework to the button handler
	warpmac_setMiddleButtonHandler(buttonHandler);
	//Set the current frequency band
	warpmac_setChannel(GHZ_2,8);
	//Allow Ethernet interrupts to occur
	warpmac_enableEthernetInterrupt();
	while(1){
	}
}




