/*! \file warpphy.h
\brief Header file for the WARPPHY functions

@version 3.0 (SVN rev 720)
@author Chris Hunter

This header file contains the macros, function prototypes, and typedefs required for WARPPHY.
*/

//WARPPHY Interface
//v3.0 alpha (SVN rev 720)
/***************CHANGELOG*****************

******************************************/
/*****************WARPPHY*****************
Description: This file specifies the
interface between to the PHY.
******************************************/

#ifndef WARPPHY_H
#define WARPPHY_H

#define OFDM_BASEADDR XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR
#define OFDM_PKTBUFF_BASEADDR XPAR_PLB_BRAM_IF_CNTLR_2_BASEADDR

//Masks for configuring modulation settings in the PHY
//Each is 8 copies of a 4-bit modulation value
#define MODMASK_BPSK 0x11111111
#define MODMASK_QPSK 0x22222222
#define MODMASK_16QAM 0x44444444
#define MODMASK_64QAM 0x66666666

#define NUMPKTBUFFS 4

//Bit masks for the options configured in Rx_ControlBits
#define RESET_BER 			0x1
#define REQ_LONG_CORR		0x2
#define DYNAMC_PKT_LENGTHS	0x4
#define BIG_PKTBUF_MODE		0x8
#define RX_SISO_MODE		0x10
#define REQ_TWO_LONG_CORR	0x20
#define REQ_SHORT_CORR		0x40
#define EXT_PKT_DETECT 		0x80
#define INT_PKT_DETECT 		0x100
#define BYPASS_CARR_REC		0x200
#define COARSE_CFO_EN		0x400
#define DEBUG_CFO_OUTSEL	0x800
#define CFO_USE_LONGCORR	0x1000
#define USE_PILOT_ARCTAN	0x2000
#define EQ_BYPASS_DIVISION  0x4000
#define SIMPLE_DYN_MOD_EN	0x10000
#define SWITCHING_DIV_EN	0x20000
#define SISO_ON_ANTB		0x40000

#define RX_GLOBAL_RESET		0x80000000

#define INTR_BAD_PKTS		0x20
#define INTR_GOOD_PKTS		0x10

//Define which radios get used below
//RADIOx_ADDR are defined by the radio controller driver
#define FIRST_RADIO RADIO2_ADDR
#define SECOND_RADIO RADIO3_ADDR

#define GHZ_5 0
#define GHZ_2 1

//Macros
#define warpphy_getBuffAddr(c) OFDM_PKTBUFF_BASEADDR + (c < NUMPKTBUFFS)*c*(0x800)

#define mimo_ofdmRx_setByteNums(c) ofdm_txrx_mimo_WriteReg(OFDM_BASEADDR, ofdm_txrx_mimo_Rx_pktByteNums_OFFSET, c)

#define mimo_ofdmRx_setFFTScaling(c) ofdm_txrx_mimo_WriteReg_TxRx_FFT_Scaling(OFDM_BASEADDR, ((ofdm_txrx_mimo_ReadReg_TxRx_FFT_Scaling(OFDM_BASEADDR) & 0x0003F) | ( (c&0x3F)<<6)))

#define mimo_ofdmRx_setRxScaling(c) *((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_mimo_Rx_Constellation_Scaling_OFFSET)) = c

#define mimo_ofdmRx_setLongCorrParams(c) *((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_mimo_Rx_PktDet_LongCorr_Params_OFFSET)) = c
#define mimo_ofdmRx_setNumOFDMSyms(c) *((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_mimo_Rx_OFDM_SymbolCounts_OFFSET))= c

#define warpphy_copyBytesFromPhy(buff,dest,len) memcpy((dest), OFDM_PKTBUFF_BASEADDR + buff * 0x1000, (len))

#define mimo_ofdmRx_setFFTWindowOffset(c) ofdm_txrx_mimo_WriteReg(OFDM_BASEADDR, ofdm_txrx_mimo_Rx_PktDet_Delay_OFFSET, (ofdm_txrx_mimo_ReadReg(OFDM_BASEADDR, ofdm_txrx_mimo_Rx_PktDet_Delay_OFFSET) & 0xFFFFE07F) | ((c&0x3F)<<7))
#define mimo_ofdmRx_setPktDetDly(c) ofdm_txrx_mimo_WriteReg(OFDM_BASEADDR, ofdm_txrx_mimo_Rx_PktDet_Delay_OFFSET, (ofdm_txrx_mimo_ReadReg(OFDM_BASEADDR, ofdm_txrx_mimo_Rx_PktDet_Delay_OFFSET) & 0xFFFFFF80)| (c&0x7F))

//#define mimo_ofdmRx_setCFO_A_KP(c) ofdm_txrx_mimo_WriteReg_Rx_FreqOffFilt_A_KP(OFDM_BASEADDR, c)

#define mimo_ofdmRx_setCFO_B_KI(c) ofdm_txrx_mimo_WriteReg_Rx_FreqOffFilt_B_KI(OFDM_BASEADDR, c)
#define mimo_ofdmRx_setCFO_B_KP(c) ofdm_txrx_mimo_WriteReg_Rx_FreqOffFilt_B_KP(OFDM_BASEADDR, c)

#define mimo_ofdmRx_setPNTrack_K(c) ofdm_txrx_mimo_WriteReg_Rx_PhaseNoiseTrack_K(OFDM_BASEADDR, c)
#define mimo_ofdmRx_setPNTrack_Kalpha(c) ofdm_txrx_mimo_WriteReg_Rx_PhaseNoiseTrack_Kalpha(OFDM_BASEADDR, c)
#define mimo_ofdmRx_setPNTrack_Kbeta(c) ofdm_txrx_mimo_WriteReg_Rx_PhaseNoiseTrack_Kbeta(OFDM_BASEADDR, c)
#define mimo_ofdmRx_setPNTrack_Kgamma(c) ofdm_txrx_mimo_WriteReg_Rx_PhaseNoiseTrack_Kgamma(OFDM_BASEADDR, c)

#define mimo_ofdmRx_setCorrThresh(c) ofdm_txrx_mimo_WriteReg_Rx_pktDet_Tresholds(OFDM_BASEADDR, ((ofdm_txrx_mimo_ReadReg_Rx_pktDet_Tresholds(OFDM_BASEADDR) & 0xFFFFFF00) | (c&0xFF)))
#define mimo_ofdmRx_setEnergyThresh(c) ofdm_txrx_mimo_WriteReg_Rx_pktDet_Tresholds(OFDM_BASEADDR, ((ofdm_txrx_mimo_ReadReg_Rx_pktDet_Tresholds(OFDM_BASEADDR) & 0xFFF000FF) | ((c&0xFFF)<<12)))


//Bit masks for OFDM Tx options
#define TX_SISO_MODE		0x1
#define TX_RANDOM_MODE		0x2
#define TX_DISABLE_ANTB_PREAMBLE 0x4
#define TX_PILOT_SCRAMBLING 0x8
//Bits 0xF are used for 4-bit preable shift value
#define TX_SISO_ON_ANTB 0x100

//MAC2PHY Options
#define TXBLOCK			0x0
#define TXNOBLOCK			0x1

#define mimo_ofdmTx_setFFTScaling(c) ofdm_txrx_mimo_WriteReg_TxRx_FFT_Scaling(OFDM_BASEADDR, ((ofdm_txrx_mimo_ReadReg_TxRx_FFT_Scaling(OFDM_BASEADDR) & 0xFC0) | ( (c&0x3F))))
#define mimo_ofdmTx_setPilot1Index(c) *((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_mimo_Tx_Pilots_Index1_OFFSET)) = c
#define mimo_ofdmTx_setPilot1Value(c) *((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_mimo_Tx_Pilots_Value1_OFFSET)) = c
#define mimo_ofdmTx_setPilot2Index(c) *((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_mimo_Tx_Pilots_Index2_OFFSET)) = c
#define mimo_ofdmTx_setPilot2Value(c) *((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_mimo_Tx_Pilots_Value2_OFFSET)) = c
#define mimo_ofdmTx_setRandom(c) *((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_mimo_Tx_RandomPayload_ModulationSelt_OFFSET)) = c
#define mimo_ofdmTx_setNumBytes(c) *((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_mimo_Tx_NumPayloadBytes_OFFSET)) = c
#define warpphy_setNumSyms(c) *((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_mimo_Tx_OFDM_SymCounts_OFFSET)) = c

#define mimo_ofdmTx_setPktDoneReset(c) *((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_mimo_Tx_Start_Reset_Control_OFFSET)) = (c<<2)&0x4

#define mimo_ofdmTx_setPreambleScaling(c) *((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_mimo_Tx_PreambleScaling_OFFSET)) = c

#define warpphy_copyBytesToPhy(buff,src,len) memcpy(OFDM_PKTBUFF_BASEADDR + buff * 0x1000,(src),(len))

#define mimo_ofdmTx_setControlBits(c) ofdm_txrx_mimo_WriteReg_Tx_ControlBits(OFDM_BASEADDR, c)
#define mimo_ofdmTx_getOptions() ofdm_txrx_mimo_ReadReg_Tx_ControlBits(OFDM_BASEADDR)



///Structure contains PHY header
typedef struct {
	///Full-rate modulation order
	unsigned char fullRate;
	///Reserved byte
	unsigned char reserved4;
	///The length of the packet (in bytes). NOTE: This should only specify the length of the payload to-be-sent.
	unsigned short int length;
	///Type of packet this particular Macframe corresponds to (e.g. DATA, ACKPACKET, etc.)
	unsigned char pktType;
	///Destination MAC address.
	unsigned char destAddr[6];
	///Source MAC address.
	unsigned char srcAddr[6];
	///Number of times this packet has been retransmitted.
	unsigned char currReSend;
	///Reserved byte
	unsigned char reserved1;
	///Reserved byte
	unsigned char reserved2;
	///Reserved byte
	unsigned char reserved3;
	///Checksum of the packet will be automatically inserted by PHY
	unsigned short int checksum;
} phyHeader __attribute__((__aligned__(8)));




int warpphy_init();
void warpphy_clearRxInterrupts();
void warpphy_pktTx(unsigned int block);
void mimo_ofdmRx_enable();
void mimo_ofdmRx_disable();
void mimo_ofdmRx_setOptions(unsigned int someOptions, unsigned int intType);
unsigned int mimo_ofdmRx_getOptions();
void mimo_ofdmTx_disable();
void mimo_ofdmTx_enable();
void warpphy_setBuffs(unsigned char txBufOffset, unsigned char rxBufOffset);
void warpphy_enableSisoMode();
void warpphy_enableMimoMode();
void warpphy_setNumTrainingSyms(unsigned int c);
void warpphy_setPktDlyPlus();
void warpphy_setPktDlyMinus();
void warpphy_set_PN_KPlus(unsigned int increment);
void warpphy_set_PN_KMinus(unsigned int decrement);
void warpphy_set_CFODebugOutput(unsigned char outputSel);
void print_CFO_constants ();
void warpphy_set_B_KPPlus(unsigned int increment);
void warpphy_set_B_KPMinus(unsigned int decrement);
void warpphy_set_B_KIPlus(unsigned int increment);
void warpphy_set_B_KIMinus(unsigned int decrement);
void warpphy_set_FFTOffset_Plus();
void warpphy_set_FFTOffset_Minus();
void warpphy_setNoiseTargetPlus();
void warpphy_setNoiseTargetMinus();
void warpphy_setTargetPlus();
void warpphy_setTargetMinus();
void warpphy_set_modulation(unsigned char baseRate, unsigned char antAFullRate, unsigned char antBFullRate);
void warpphy_setSISOAntenna(unsigned char antSel);
int warpphy_setChannel(unsigned char band,unsigned int c);
int warpphy_applyTxDCOCorrection(unsigned int radioSelection);





#endif

