//Chris Hunter
/******************HOPMAC*****************
This exercise is an extension of the previous
UNIMAC exercise. In fact, you are welcome
top copy and paste the parts of the code you
completed in that exercise for the similar
tasks here.
 
The twist in this lab is that the server
will periodically hop to a new part of the
spectrum. Before doing so, it will transmit
a special control packet (HOPPACKET) that
contains the destination frequency band.
 
If the packet is missed, your receiver will
be stuck until the transmitter randomly hops
back onto the channel you were sitting at.
This can potentially be a very long time.
An advanced extension to this lab is to
scan for packets in various channels when
no good packet is received for a certain
amount of time. For our purposes, let this
timeout be around 200ms.
 *****************************************/

#include "warpmac.h"
#include "warpphy.h"
#include "xparameters.h"
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include "xutil.h"
#include "xcache_l.h"
#include "xexception_l.h"
#include "xcache_l.h"
#include "xutil.h"
#include "xexception_l.h"
#include "ofdm_timer.h"

//Routing table with agreed upon mapping between dipswitches and physical addresses
typedef struct {
	unsigned char addr[6];
} route;

//Byte array containing physical MAC address of this node
unsigned char myAddr[6];
//Instance of the routing table
route routeTable[16];
//Index to the routing table that identifies this node
unsigned char myID;

//Data packet with payload meant for Ethernet transmission
#define DATAPACKET 0
//Acknowedgement packet meant for halting retransmissions
#define ACKPACKET 1
//Control packet containing destination frequency band
#define HOPPACKET 2


unsigned char chan = 8;

//Buffer for holding a packet-to-xmit across multiple retransmissions
Macframe txBuffer; 
//Buffer to hold received packet
Macframe rxBuffer;

/////////////////////SEEK MODE: OPTIONAL EXTENSION//////////////////////////
unsigned int waitTime = 200;	//If a packet hasn't been received in the last 200 ms, exhaustively search the spectrum
unsigned char scanMode=0; //By default, the optional scan mode is disabled
void middlebutton(){
	/*Disables the optional scan mode*/
	scanMode=0;
}

void upbutton(){
	/*Enables the optional scan mode*/
	scanMode=1;
	warpmac_stopTimer();
	warpmac_setTimerVal(waitTime*40000);
	warpmac_startTimer(DISABLECSMA);
}

void leftbutton(){
	/*Increases the timeout to begin exhaustive scan*/
	waitTime=waitTime-10;
}

void rightbutton(){
	/*Decreases the timeout to begin exhaustive scan*/
	waitTime=waitTime+10;
}

unsigned char index = 0;
int timerExpire(){
		if(scanMode){
			chan = (rand()%14)+1;
			warpmac_setChannel(GHZ_2,chan);
		    warpmac_leftHexBit((index++)%7);
			warpmac_stopTimer();
			warpmac_setTimerVal(waitTime*40000);
			warpmac_startTimer(DISABLECSMA);
		}
}
/////////////////////////////////////////////////////////////////////////////

int ethernet_callback(Xuint32 length, char* payload){
	/*This function is attached to the underlying framework in the main
	 function of this file. It is executed upon the reception of an Ethernet
	 frame. By the time this function is called, the payload of the Ethernet
	 frame is already being copied to the OFDM PHY transmiter's packet buffer
	 via the internal DMA engine in the Ethernet MAC. All the user is responsible
	 for is creating a header for the over-the-air packet. The length of the received
	 Ethernet frame is passed into this function as an argument*/
	
	txBuffer.header.length = length;
	txBuffer.header.pktType = DATAPACKET;
	txBuffer.header.fullRate = QPSK;
	memcpy(txBuffer.header.destAddr,routeTable[0].addr,6);
	memcpy(txBuffer.header.srcAddr,myAddr,6);
	warpmac_sendOfdm(&txBuffer);
return 0;
}


int receiveBadPacket(Macframe* packet){
	/*This function is attached to the underlying framework in the main
	 function of this file. It is executed when a packet is received from the
	 OFDM PHY receiver that fails checksum. Since at least one bit was in error,
	 the packet is, in general, no good to us. We use this function to blink
	 LEDs to help us visualize subjectively the good/bad ratio of received
	 packets. Additionally, this function could be used to keep track of the
	 number of failed packets more objectively for rate adaptation schemes.*/
	
	warpmac_incrementLEDLow();
}

int receiveGoodPacket(Macframe* packet) {
	/////////////////////SEEK MODE: OPTIONAL EXTENSION//////////////////////////
	warpmac_stopTimer();
	warpmac_setTimerVal(waitTime*40000);
	warpmac_startTimer(DISABLECSMA);
	/////////////////////////////////////////////////////////////////////////////
	
	warpmac_incrementLEDHigh();
	
	warpmac_leftHex(chan);
	if(warpmac_addressedToMe(packet)){
		Macframe ackPacket;
		ackPacket.header.length = 0;
		ackPacket.header.pktType = ACKPACKET;
		ackPacket.header.fullRate = QPSK;
		memcpy(ackPacket.header.srcAddr,myAddr,6);
		memcpy(ackPacket.header.destAddr,packet->header.srcAddr,6);
		warpmac_setTxBuffer(2);
		warpmac_sendOfdm(&ackPacket);
		warpmac_setTxBuffer(1);
		packet->isNew = 1;
		warpmac_sendEthernet(packet);
		warpmac_phyInterruptClear();
		packet->isNew = 0;
	}
	
	
	else {
		
		//Check if the received packet's destination address was the broadcast address
		//Check if the packet type was a HOPPACKET
		//Switch to the channel in the reserved1 header field
		
		unsigned char broadcast[6] = {255,255,255,255,255,255};
		unsigned char sum = 0;
		unsigned char i;
		for(i=0;i<6;i++){
			sum = sum + (packet->header.destAddr[i]==broadcast[i]);
		}
		if(sum==6){
			if(packet->header.pktType==HOPPACKET){
				warpmac_setChannel(GHZ_2,packet->header.reserved1);
				chan = packet->header.reserved1;
				warpmac_leftHex(chan);
			}
		}
	}	
	

return 0;
}





int main(){

	//Read Dip Switch value from FPGA board.
	//This value will be used as an index into the routing table for other nodes
	myID = warpmac_getMyId();
	
	//Create an arbitrary address for this node
	unsigned char tmpAddr[6] = {0x16,0x24,0x63,0x53,0xe2,0xc2+myID};
	
	//Copy this address into a global array for the rest of the code to access
	memcpy(myAddr,tmpAddr,6);
	
	//Fill an arbitrary routing table so that nodes know each others' addresses
	unsigned char i;
	for(i=0;i<16;i++){
		routeTable[i].addr[0] = myAddr[0];
		routeTable[i].addr[1] = myAddr[1];
		routeTable[i].addr[2] = myAddr[2];
		routeTable[i].addr[3] = myAddr[3];
		routeTable[i].addr[4] = myAddr[4];
		routeTable[i].addr[5] = myAddr[5]+i-myID;
	}
	
	//Initialize the framework
	warpmac_init();
	//Tell the framework the MAC address of this node
	warpmac_setMacAddr(&myAddr);
	//Tell the framework where to place a received packet
	warpmac_setRxBuffer(&rxBuffer,0);
	//Tell the framework where to send a received packet from
	warpmac_setTxBuffer(1);
 	//Copy MAC address into source field
	memcpy(txBuffer.header.srcAddr,myAddr,6);
	//Attach the framework to the good packet handler
	warpmac_setGoodPacketHandler(receiveGoodPacket);
	//Attach the framework to the bad packet handler
	warpmac_setBadPacketHandler(receiveBadPacket);
	//Attach the framework to the ethernet handler
	warpmac_setEmacHandler(ethernet_callback);
	//Set the current frequency band
	xil_printf("Channel = %d\r\n",warpmac_setChannel(GHZ_2,12));
	//Allow Ethernet interrupts to occur
	warpmac_enableEthernetInterrupt();
	
	/////////////////////SEEK MODE: OPTIONAL EXTENSION//////////////////////////
	warpmac_setMiddleButtonHandler(middlebutton);
	warpmac_setUpButtonHandler(upbutton);
	warpmac_setLeftButtonHandler(leftbutton);
	warpmac_setRightButtonHandler(rightbutton);
	warpmac_setTimerHandler(timerExpire);
	warpmac_setTimerVal(waitTime*40000);
	warpmac_startTimer(DISABLECSMA);
	/////////////////////////////////////////////////////////////////////////////
	
	while(1){
	}
}




