//Chris Hunter
/******************NOMAC*****************
The dumbest MAC. Forwards all traffic from
ethernet to the radio medium and vice versa.
No address filtering... no retransmissions.
Medium access is completely dependent on
the source traffic, so there is no actual
medium access control. Hence, we have noMac.
*****************************************/

#include "warpmac.h"
#include "warpphy.h"
#include "xparameters.h"
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include "xutil.h"
#include "xcache_l.h"
#include "xexception_l.h"
#include "xcache_l.h"
#include "xutil.h"
#include "xexception_l.h"


//Buffer for holding a packet-to-xmit across multiple retransmissions
Macframe txBuffer; 
//Buffer to hold received packet
Macframe rxBuffer;

int ethernet_callback(Xuint32 length){
	/*This function is attached to the underlying framework in the main
	 function of this file. It is executed upon the reception of an Ethernet
	 frame. By the time this function is called, the payload of the Ethernet
	 frame is already being copied to the OFDM PHY transmiter's packet buffer
	 via the internal DMA engine in the Ethernet MAC. All the user is responsible
	 for is creating a header for the over-the-air packet. The length of the received
	 Ethernet frame is passed into this function as an argument*/
	
	
	//PSEUDOCODE:
	//1) Fill in the length of the Ethernet frame into the length field of the header of the global transmit buffer.
	//		(for structure definitions, see http://warp.rice.edu/WARP_API/annotated.html in the overall WARP_API)
	//2) Set the modulation order of the packet by filling in the appropriate field in the header. Use QPSK.
	//		(#define can be found in the warpmac.h documentation here: http://warp.rice.edu/WARP_API/warpmac_8h.html#a3 )
	//3) Send the transmit buffer over OFDM
	//		(Check out the CSMAMAC source for reference here: http://warp.rice.edu/trac/browser/ResearchApps/MAC/CSMAMAC/csmaMac.c )
	
	txBuffer.header.length = length;
	txBuffer.header.fullRate = QPSK;
	warpmac_sendOfdm(&txBuffer);
	return 0;
}


int receiveBadPacket(Macframe* packet){
	/*This function is attached to the underlying framework in the main
	 function of this file. It is executed when a packet is received from the
	 OFDM PHY receiver that fails checksum. Since at least one bit was in error,
	 the packet is, in general, no good to us. We use this function to blink
	 LEDs to help us visualize subjectively the good/bad ratio of received
	 packets. Additionally, this function could be used to keep track of the
	 number of failed packets more objectively for rate adaptation schemes.*/
	
	warpmac_incrementLEDLow();
}

int receiveGoodPacket(Macframe* packet) {
	/*This function is attached to the underlying framework in the main
	 function of this file. It is executed when a packet is received from the
	 OFDM PHY receiver that passes checksum. This function is analogous to the
	 ethernet_callback function, but in the other direction. It pushes the
	 received packet over-the-wire to its sink.*/
	
	warpmac_incrementLEDHigh();
	
	//PSEUDOCODE:
	//1) Send the transmit buffer over Ethernet.
	
	warpmac_sendEthernet(packet);
	return 0;
}


int main(){
	//Initialize the framework
	warpmac_init();
	//Tell the framework where to place a received packet
	warpmac_setRxBuffer(&rxBuffer,0);
	//Tell the framework where to send a received packet from
	warpmac_setTxBuffer(1);
	//Set the current frequency band
	warpmac_setChannel(GHZ_2,8);
	//Attach the framework to the good packet handler
	warpmac_setGoodPacketHandler(receiveGoodPacket);
	//Attach the framework to the bad packet handler
	warpmac_setBadPacketHandler(receiveBadPacket);
	//Attach the framework to the ethernet handler
	warpmac_setEmacHandler(ethernet_callback);
	//Allow Ethernet interrupts to occur
	warpmac_enableEthernetInterrupt();
	while(1){
	}
}




